

import re

from ..tokenization import FullTokenizer

from ..utils import (
    get_or_make_label_encoder, BOS_TOKEN, EOS_TOKEN)
from ..create_generators import create_pretraining_generator, create_single_problem_generator

from .ner_data import gold_horse_ent_type_process_fn, read_ner_data
from .preproc_decorator import proprocessing_fn


@proprocessing_fn
def weibo_fake_cls(params, mode):
    """Just a test problem to test multiproblem support

    Arguments:
        params {Params} -- params
        mode {mode} -- mode
    """
    data = read_ner_data(file_pattern='data/ner/weiboNER*',
                         proc_fn=gold_horse_ent_type_process_fn)
    if mode == 'train':
        data = data['train']
    else:
        data = data['eval']
    inputs_list = data['inputs'][:100]
    target_list = data['target'][:100]

    new_target_list = ['1' if len(set(t)) > 1 else '0' for t in target_list]

    return inputs_list, new_target_list


@proprocessing_fn
def weibo_fake_seq2seq_tag(params, mode: str):

    data = read_ner_data(file_pattern='data/ner/weiboNER*',
                         proc_fn=gold_horse_ent_type_process_fn)
    if mode == 'train':
        data = data['train']
    else:
        data = data['eval']
    inputs_list = data['inputs'][:100]
    target_list = data['target'][:100]
    new_target_list = [['1', '2'] for t in target_list]
    return inputs_list, new_target_list


def weibo_pretrain(params, mode):

    sentence_split = r'[.!?。？！]'

    tokenizer = FullTokenizer(vocab_file=params.vocab_file)
    data = read_ner_data(file_pattern='data/ner/weiboNER*',
                         proc_fn=gold_horse_segment_process_fn)
    if mode == 'train':
        data = data['train']
    else:
        data = data['eval']
    inputs_list = data['inputs']

    segmented_list = []
    for document in inputs_list:
        segmented_list.append([])
        doc_string = ''.join(document)
        splited_doc = re.split(sentence_split, doc_string)
        for sentence in splited_doc:
            if sentence:
                segmented_list[-1].append(list(sentence))
    segmented_list = [doc for doc in segmented_list if doc]

    return create_pretraining_generator('weibo_pretrain',
                                        segmented_list,
                                        None,
                                        None,
                                        params,
                                        tokenizer,
                                        mode)


@proprocessing_fn
def weibo_fake_seq_tag(params, mode):
    data = read_ner_data(file_pattern='data/ner/weiboNER*',
                         proc_fn=gold_horse_ent_type_process_fn)
    if mode == 'train':
        data = data['train']
    else:
        data = data['eval']
    inputs_list = data['inputs'][:100]
    target_list = data['target'][:100]

    return inputs_list, target_list
